@@ -1,5 +1,5 @@
 #
-# $Id: Layer.pm 347 2012-01-14 08:50:01Z gomor $
+# $Id: Layer.pm 357 2014-04-08 13:34:04Z gomor $
 #
 package Net::Frame::Layer;
 use strict;
@@ -95,9 +95,53 @@ sub dump { CORE::unpack('H*', shift->raw) }
 # Useful subroutines
 #
 
-use Socket;
-use Socket6 qw(NI_NUMERICHOST NI_NUMERICSERV inet_pton inet_ntop getaddrinfo
-   getnameinfo);
+# load AF_INET and default imports from Socket.  Safe back to at least 5.8.8
+use Socket qw(:DEFAULT AF_INET);
+BEGIN {
+    # imports that may or may not be in Socket.
+    my @imports = (
+        qw(AF_INET6 NI_NUMERICHOST NI_NUMERICSERV getaddrinfo getnameinfo));
+    my @socket6_imports;
+
+    # This might be overkill, but I'm not certain that all these imports
+    # were added to Socket at the same time.
+    for my $export (@imports) {
+        eval { Socket->import($export); };
+        if ($@) {
+            push @socket6_imports, $export;
+        }
+    }
+
+    eval {
+        # Test to see if the sub works.
+        # Socket::inet_ntop() may exist, but die with:
+        # Socket::inet_ntop not implemented on this architecture
+        Socket::inet_ntop( AF_INET, "\0\0\0\0" );    # test
+        Socket->import('inet_ntop');    # import if the test doesn't die
+    };
+    if ($@) {
+        push @socket6_imports, 'inet_ntop';
+    }
+    eval {
+        # Test to see if the sub works.
+        # Socket::inet_pton() may exist, but die with:
+        # Socket::inet_pton not implemented on this architecture
+        Socket::inet_pton( AF_INET, '0.0.0.0' );    # test
+        Socket->import('inet_pton');    # import if the test doesn't die
+    };
+    if ($@) {
+        push @socket6_imports, 'inet_pton';
+    }
+
+    if (@socket6_imports) {
+
+        # something we want wasn't found in Socket time to try Socket6
+        eval { require Socket6 };
+        die $@ if $@;
+        Socket6->import(@socket6_imports);
+    }
+}
+
 require Net::IPv6Addr;
 
 sub getHostIpv4Addr {
@@ -130,7 +174,7 @@ sub getHostIpv6Addr {
    return undef unless $name;
    return $name if Net::IPv6Addr::is_ipv6($name);
 
-   my @res = getaddrinfo($name, 'ssh', Socket6::AF_INET6(), SOCK_STREAM);
+   my @res = getaddrinfo($name, 'ssh', AF_INET6, SOCK_STREAM);
    if (@res >= 5) {
       my ($ipv6) = getnameinfo($res[3], NI_NUMERICHOST | NI_NUMERICSERV);
       $ipv6 =~ s/%.*$//;
@@ -142,10 +186,10 @@ sub getHostIpv6Addr {
    undef;
 }
 
-sub inetAton  { inet_aton(shift())                    }
-sub inetNtoa  { inet_ntoa(shift())                    }
-sub inet6Aton { inet_pton(Socket6::AF_INET6(), shift()) }
-sub inet6Ntoa { inet_ntop(Socket6::AF_INET6(), shift()) }
+sub inetAton  { inet_aton(shift()) }
+sub inetNtoa  { inet_ntoa(shift()) }
+sub inet6Aton { inet_pton(AF_INET6, shift()) }
+sub inet6Ntoa { inet_ntop(AF_INET6, shift()) }
 
 sub getRandomHighPort {
    my $highPort = int rand 0xffff;
@@ -157,10 +201,7 @@ sub getRandom32bitsInt { int rand 0xffffffff }
 sub getRandom16bitsInt { int rand 0xffff     }
 
 sub convertMac {
-   my ($mac) = @_;
-   $mac =~ s/(..)/$1:/g;
-   $mac =~ s/:$//;
-   lc($mac);
+   return lc(join(':', $_[0] =~ /../g));
 }
 
 sub inetChecksum {
@@ -171,7 +212,8 @@ sub inetChecksum {
    my $nshort   = $len / 2;
    my $checksum = 0;
    $checksum   += $_ for CORE::unpack("S$nshort", $phpkt);
-   $checksum   += CORE::unpack('C', substr($phpkt, $len - 1, 1)) if $len % 2;
+   # XXX: This line never does anything as the lenth was made even above. Currently testing it breaks nothing.
+   #$checksum   += CORE::unpack('C', substr($phpkt, $len - 1, 1)) if $len % 2;
    $checksum    = ($checksum >> 16) + ($checksum & 0xffff);
 
    CORE::unpack('n',
@@ -333,7 +375,7 @@ Patrice E<lt>GomoRE<gt> Auffret
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright (c) 2006-2012, Patrice E<lt>GomoRE<gt> Auffret
+Copyright (c) 2006-2014, Patrice E<lt>GomoRE<gt> Auffret
 
 You may distribute this module under the terms of the Artistic license.
 See LICENSE.Artistic file in the source distribution archive.
